package org.ahmadsoft.arrays;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

public class ArraysTest {

	private static final int iterations = 3000;
	private static final int capacity = 2500;
	private static final long randomSeed = 42342348881l;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		PrintStream out = System.out;
		out.println("Arrays Tester v1.0");
		out.println(System.getProperty("java.runtime.name"));
		out.println(System.getProperty("java.runtime.version"));
		
		// initialize array
		//
		ArrayList<Integer> sunArrayList = new ArrayList<Integer>(capacity);
		ArrayList2<Integer> asoftArrayList2 = new ArrayList2<Integer>(capacity); 
		ArrayList3<Integer> asoftArrayList3 = new ArrayList3<Integer>(capacity); 
		
		System.out.println("testing java.util.ArrayList");
		testHead(sunArrayList);
		testTail(sunArrayList);
		testEmpty(sunArrayList);
		testEveryOther(sunArrayList);
		testRandom(sunArrayList);
		
		System.out.println("testing org.ahmadsoft.arrays.ArrayList2");
		testHead(asoftArrayList2);
		testTail(asoftArrayList2);
		testEmpty(asoftArrayList2);
		testEveryOther(asoftArrayList2);
		testRandom(asoftArrayList2);
		
		System.out.println("testing org.ahmadsoft.arrays.ArrayList3");
		testHead(asoftArrayList3);
		testTail(asoftArrayList3);
		testEmpty(asoftArrayList3);
		testEveryOther(asoftArrayList3);
		testRandom(asoftArrayList3);
	}
	
	public static void testRandom(final List<Integer> l) {
		Random r = new Random(randomSeed);
		final int removeCount = 150;
		final HashSet<Integer> forRemoval = new HashSet<Integer>();
		for (int j=0; j<removeCount; ++j)
			forRemoval.add(r.nextInt() % capacity);
		System.out.print("  removing "+forRemoval.size()+" random items from list (size=" + capacity + ")");
		
		long tsum = 0;
		for (int j=0; j<iterations; ++j) {
			l.clear();
			for (int k=0;k<capacity;++k)
				l.add(k);
			
			tsum -= System.nanoTime();
			l.removeAll(forRemoval);
			tsum += System.nanoTime();
		}
		System.out.println(" average time=" + (tsum/iterations) + "ns.");
	}
	
	public static void testHead(final List<Integer> l) {
		final int removeCount = 150;
		System.out.print("  removing "+removeCount+" from head of list (size=" + capacity + ")");
		final HashSet<Integer> forRemoval = new HashSet<Integer>();
		for (int j=0; j<removeCount; ++j)
			forRemoval.add(j);
		
		long tsum = 0;
		for (int j=0; j<iterations; ++j) {
			l.clear();
			for (int k=0;k<capacity;++k)
				l.add(k);
			
			tsum -= System.nanoTime();
			l.removeAll(forRemoval);
			tsum += System.nanoTime();
		}
		System.out.println(" average time=" + (tsum/iterations) + "ns.");
	}
	
	public static void testEveryOther(final List<Integer> l) {
		final int removeCount = 150;
		System.out.print("  removing "+removeCount+" from interleaved every other with the first "  + (removeCount*2) + " of list (size=" + capacity + ")");
		final HashSet<Integer> forRemoval = new HashSet<Integer>();
		for (int j=0; j<removeCount; ++j)
			forRemoval.add(j*2);
		
		long tsum = 0;
		for (int j=0; j<iterations; ++j) {
			l.clear();
			for (int k=0;k<capacity;++k)
				l.add(k);
			
			tsum -= System.nanoTime();
			l.removeAll(forRemoval);
			tsum += System.nanoTime();
		}
		System.out.println(" average time=" + (tsum/iterations) + "ns.");
	}
	
	public static void testTail(final List<Integer> l) {
		final int removeCount = 150;
		System.out.print("  removing "+removeCount+" from tail of list (size=" + capacity + ")");
		final HashSet<Integer> forRemoval = new HashSet<Integer>();
		for (int j=0; j<removeCount; ++j)
			forRemoval.add(capacity-removeCount+j);
		
		long tsum = 0;
		for (int j=0; j<iterations; ++j) {
			l.clear();
			for (int k=0;k<capacity;++k)
				l.add(k);
			
			tsum -= System.nanoTime();
			l.removeAll(forRemoval);
			tsum += System.nanoTime();
		}
		System.out.println(" average time=" + (tsum/iterations) + "ns.");
	}
	
	public static void testEmpty(final List<Integer> l) {
		System.out.print("  removing nothing from list (size=" + capacity + ")");
		final HashSet<Integer> forRemoval = new HashSet<Integer>();
		
		long tsum = 0;
		for (int j=0; j<iterations; ++j) {			
			tsum -= System.nanoTime();
			l.removeAll(forRemoval);
			tsum += System.nanoTime();
		}
		System.out.println(" average time=" + (tsum/iterations) + "ns.");
	}

}
